@@ -4,6 +4,7 @@ import base64
 import json
 import os
 import uuid
+import warnings
 from collections import OrderedDict
 from concurrent.futures import Future, ThreadPoolExecutor
 from datetime import datetime
@@ -288,7 +289,7 @@ def _extract_df_columns(
         df = df.select(list(expected_cols.values()))
 
     elif version == DatasetVersion.COMPUTE_RESOLVER_OUTPUT_V1:
-        unique_features = set(df.select(pl.col("feature_name").unique())["feature_name"].to_list())
+        unique_features = set(df.select(pl.col("feature_name").unique()).lazy().collect()["feature_name"].to_list())
         cols = [
             pl.col("value").filter(pl.col("feature_name") == fqn).first().alias(cast(str, fqn))
             for fqn in unique_features
@@ -455,6 +456,11 @@ class DatasetRevisionImpl(DatasetRevision):
 
     @property
     def data_as_polars(self) -> pl.LazyFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `DatasetRevision.data_as_polars` is deprecated. Please use the method `DatasetRevision.get_data_as_polars()` instead."
+            )
+        )
         return self.get_data_as_polars()
 
     def get_data_as_polars(
@@ -475,6 +481,11 @@ class DatasetRevisionImpl(DatasetRevision):
 
     @property
     def data_as_pandas(self) -> pd.DataFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `DatasetRevision.data_as_pandas` is deprecated. Please use the method `DatasetRevision.get_data_as_pandas()` instead."
+            )
+        )
         return self.get_data_as_pandas()
 
     def get_data_as_pandas(
@@ -500,6 +511,11 @@ class DatasetRevisionImpl(DatasetRevision):
 
     @property
     def data_as_dataframe(self) -> DataFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `DatasetRevision.data_as_dataframe` is deprecated. Please use the method `DatasetRevision.get_data_as_dataframe()` instead."
+            )
+        )
         return self.get_data_as_dataframe()
 
     def get_data_as_dataframe(
@@ -588,7 +604,7 @@ class DatasetImpl(Dataset):
         return self._version
 
     @property
-    def revisions(self) -> List[DatasetRevisionImpl]:
+    def revisions(self) -> Sequence[DatasetRevisionImpl]:
         return self._revisions
 
     @property
@@ -605,6 +621,11 @@ class DatasetImpl(Dataset):
 
     @property
     def data_as_polars(self) -> pl.LazyFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `Dataset.data_as_polars` is deprecated. Please use the method `Dataset.get_data_as_polars()` instead."
+            )
+        )
         return self.get_data_as_polars()
 
     def get_data_as_polars(
@@ -621,6 +642,11 @@ class DatasetImpl(Dataset):
 
     @property
     def data_as_pandas(self) -> pd.DataFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `Dataset.data_as_pandas` is deprecated. Please use the method `Dataset.get_data_as_pandas()` instead."
+            )
+        )
         return self.get_data_as_pandas()
 
     def get_data_as_pandas(
@@ -637,6 +663,11 @@ class DatasetImpl(Dataset):
 
     @property
     def data_as_dataframe(self) -> DataFrame:
+        warnings.warn(
+            DeprecationWarning(
+                "The property `dataset.data_as_dataframe` is deprecated. Please use `dataset.get_data_as_dataframe()` instead."
+            )
+        )
         return self.get_data_as_dataframe()
 
     def get_data_as_dataframe(
