@@ -11,15 +11,6 @@ class ChalkBaseException(Exception, abc.ABC):
     look only for this exception class.
     """
 
-    message: str
-    """A message describing the specific type of exception raised."""
-
-    full_message: str
-    """A message that describes the specific type of exception raised
-    and contains the readable representation of each error in the
-    errors attribute.
-    """
-
     errors: List[ChalkError]
     """The errors from executing a Chalk operation.
 
@@ -37,10 +28,15 @@ class ChalkBaseException(Exception, abc.ABC):
 
     @property
     def message(self) -> str:
+        """A message describing the specific type of exception raised."""
         raise NotImplementedError
 
     @property
     def full_message(self) -> str:
+        """A message that describes the specific type of exception raised
+        and contains the readable representation of each error in the
+        errors attribute.
+        """
         if self.errors:
             return self.message + "\n" + "\n".join(["\t" + e.message for e in self.errors])
 
@@ -113,15 +109,17 @@ class ChalkAuthException(ChalkBaseException):
     working directory is set to the root of your project.
     """
 
-    message = (
-        "Explicit `client_id` and `client_secret` are not provided, "
-        "there is no `~/.chalk.yml` file with applicable credentials, "
-        "and the environment variables `CHALK_CLIENT_ID` and "
-        "`CHALK_CLIENT_SECRET` are not set. "
-        "You may need to run `chalk login` from your command line, "
-        "or check that your working directory is set to the root of "
-        "your project."
-    )
+    @property
+    def message(self):
+        return (
+            "Explicit `client_id` and `client_secret` are not provided, "
+            "there is no `~/.chalk.yml` file with applicable credentials, "
+            "and the environment variables `CHALK_CLIENT_ID` and "
+            "`CHALK_CLIENT_SECRET` are not set. "
+            "You may need to run `chalk login` from your command line, "
+            "or check that your working directory is set to the root of "
+            "your project."
+        )
 
 
 __all__ = [
