@@ -292,13 +292,13 @@ class FeatureResult(BaseModel):
     """Detailed information about how this feature was computed."""
 
 
-class _ExchangeCredentialsRequest(BaseModel):
+class ExchangeCredentialsRequest(BaseModel):
     client_id: str
     client_secret: str
     grant_type: str
 
 
-class _ExchangeCredentialsResponse(BaseModel):
+class ExchangeCredentialsResponse(BaseModel):
     access_token: str
     token_type: str
     expires_in: int
@@ -331,7 +331,7 @@ class OnlineQueryRequest(BaseModel):
     meta: Optional[Mapping[str, str]] = None
 
 
-class _TriggerResolverRunRequest(BaseModel):
+class TriggerResolverRunRequest(BaseModel):
     resolver_fqn: str
 
 
