@@ -6,6 +6,7 @@ from typing import Any, Callable, Dict, Optional, Union
 from executing import Source
 
 from chalk.features import Filter
+from chalk.features._chalkop import op
 
 
 class ExecutingException(Exception):
@@ -55,6 +56,23 @@ def _get_func_frame_and_nodes(condition: Callable[[Optional[ast.AST]], bool]):
     raise RuntimeError("Condition not found in stack")
 
 
+def parse_feature_iter(f):
+    func_frame, func_node = _get_func_frame_and_nodes(lambda node: isinstance(node, ast.Call))
+    if not isinstance(func_node, ast.Call):
+        raise RuntimeError("Could not evaluate function")
+    if not isinstance(func_node.func, ast.Name):
+        raise RuntimeError("Could not evaluate function")
+    func_name = func_node.func.id
+    if func_name == "sum":
+        return op.sum(f)
+    elif func_name == "max":
+        return op.max(f)
+    elif func_name == "min":
+        return op.min(f)
+
+    raise RuntimeError(f"Could not evaluate function {func_name}")
+
+
 def parse_dataframe_getitem():
     func_frame, func_node = _get_func_frame_and_nodes(
         lambda node: isinstance(node, ast.Subscript) and not isinstance(node.slice, ast.Name)
