@@ -458,7 +458,8 @@ class DataFrameImpl(metaclass=DataFrameMeta):
                 # Convert the columns to root fqn strings
                 new_data_dict: Dict[str, Union[Sequence[Any], pl.Series]] = {}
                 for k, v in data.items():
-                    feature = Feature.from_root_fqn(str(k))
+                    str_k = str(k)
+                    feature = Feature.from_root_fqn(str_k)
 
                     if feature.is_has_one or feature.is_has_many:
                         name = "has-one" if feature.is_has_one else "has-many"
@@ -481,17 +482,22 @@ class DataFrameImpl(metaclass=DataFrameMeta):
                                     missing_value_strategy=missing_value_strategy,
                                 )
                             except (TypeError, ValueError) as e:
+                                if len(v) > 0:
+                                    type_error = f" The first value that could not be loaded has type '{type(v[0])}'."
+                                else:
+                                    type_error = ""
+
                                 raise TypeError(
                                     (
                                         f"The values for feature `{k}` could not be loaded into a DataFrame column "
-                                        f"of type `{feature.converter.pyarrow_dtype}`"
+                                        f"of type `{feature.converter.pyarrow_dtype}`." + type_error
                                     )
                                 ) from e
                             series = pl.from_arrow(pa_array)
                             assert isinstance(series, pl.Series)
                         else:
                             series = v
-                    new_data_dict[str(k)] = series
+                    new_data_dict[str_k] = series
 
                 data = new_data_dict
             data = pl.DataFrame(data)
@@ -502,9 +508,11 @@ class DataFrameImpl(metaclass=DataFrameMeta):
         # Rename / validate that all column names are root fqns
         if self._pydantic_model is None:
             self.columns = tuple(Feature.from_root_fqn(str(c)) for c in underlying.columns)
-            underlying = underlying.rename(
-                {original_c: new_c.root_fqn for (original_c, new_c) in zip(underlying.columns, self.columns)}
-            )
+            # n.b. the goal here is to normalize "feature" and "str" columns to "str" columns
+            if any(not isinstance(c, str) for c in underlying.columns):
+                underlying = underlying.rename(
+                    {original_c: new_c.root_fqn for (original_c, new_c) in zip(underlying.columns, self.columns)}
+                )
         else:
             self.columns = ()
 
@@ -585,6 +593,42 @@ class DataFrameImpl(metaclass=DataFrameMeta):
         else:
             return None
 
+    @classmethod
+    def read_delta(
+        cls: Type[TDataFrame],
+        table_uri: str,
+        version: Optional[int] = None,
+        columns: Optional[
+            Union[
+                Dict[str, Union[str, Feature, Any]],
+                Dict[int, Union[str, Feature, Any]],
+            ]
+        ] = None,
+        delta_table_options: Optional[Dict[str, Any]] = None,
+        pyarrow_options: Optional[Dict[str, Any]] = None,
+    ) -> TDataFrame:
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+
+        if columns is None:
+            cols_to_select, _, new_columns = None, None, None
+        else:
+            cols_to_select, _, new_columns = cls._parse_columns(columns)
+
+        data = pl.read_delta(
+            table_uri=table_uri,
+            version=version,
+            columns=cols_to_select,
+            storage_options=DataFrame._get_storage_options(),
+            delta_table_options=delta_table_options,
+            pyarrow_options=pyarrow_options,
+        )
+        if new_columns is not None:
+            data = data.rename({c: new_columns for c in data.columns})
+        return cls(data)
+
     @classmethod
     def read_parquet(
         cls: Type[TDataFrame],
