@@ -1,17 +1,18 @@
+from __future__ import annotations
+
 import ast
 import copy
 import functools
 import inspect
-from itertools import islice
 import re
+import sys
 import textwrap
 import threading
 import types
 from datetime import datetime, timedelta
+from itertools import islice
 from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Tuple, Type, TypeVar, Union, cast, overload
 
-import sys
-
 from chalk.features._class_property import classproperty, classproperty_support
 from chalk.features.feature_field import Feature, _VersionInfo
 from chalk.features.feature_set import Features, FeatureSetBase
@@ -21,7 +22,7 @@ from chalk.features.tag import Tags
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import Windowed
 from chalk.utils.collections import ensure_tuple
-from chalk.utils.duration import Duration, timedelta_to_duration
+from chalk.utils.duration import Duration, parse_chalk_duration
 from chalk.utils.metaprogramming import MISSING, create_fn, field_assign, set_new_attribute
 from chalk.utils.string import removeprefix, to_snake_case
 
@@ -37,8 +38,8 @@ def features(
     *,
     owner: Optional[str] = None,
     tags: Optional[Tags] = None,
-    etl_offline_to_online: bool = ...,
-    max_staleness: Optional[Duration] = ...,
+    etl_offline_to_online: bool = False,
+    max_staleness: Optional[Duration] = None,
     name: Optional[str] = None,
 ) -> Callable[[Type[T]], Type[T]]:
     ...
@@ -54,8 +55,8 @@ def features(
     *,
     owner: Optional[str] = None,
     tags: Optional[Tags] = None,
-    etl_offline_to_online: Optional[bool] = None,
-    max_staleness: Optional[Duration] = ...,
+    etl_offline_to_online: bool = False,
+    max_staleness: Optional[Duration] = None,
     name: Optional[str] = None,
 ) -> Union[Callable[[Type[T]], Type[T]], Type[T]]:
     """Chalk lets you spell out your features directly in Python.
@@ -112,6 +113,7 @@ def features(
     """
 
     def wrap(c: Type[T]) -> Type[T]:
+        nonlocal max_staleness
         namespace = name or to_snake_case(c.__name__)
         if name is not None and re.sub(r"[^a-z_0-9]", "", namespace) != namespace:
             raise ValueError(
@@ -122,14 +124,16 @@ def features(
             )
         if name is not None and len(namespace) == 0:
             raise ValueError(f"Namespace cannot be an empty string, but is for the class '{c.__name__}'.")
+        if max_staleness is None:
+            max_staleness = timedelta(0)
+        if isinstance(max_staleness, str):
+            max_staleness = parse_chalk_duration(max_staleness)
         updated_class = _process_class(
             cls=c,
             owner=owner,
             tags=ensure_tuple(tags),
             etl_offline_to_online=etl_offline_to_online,
-            max_staleness=timedelta_to_duration(max_staleness)
-            if isinstance(max_staleness, timedelta)
-            else max_staleness,
+            max_staleness=max_staleness,
             namespace=namespace,
         )
         assert issubclass(updated_class, Features)
@@ -230,7 +234,7 @@ def _init_fn(
     for from_, to in alias_from_to.items():
         body_lines.extend(
             [
-                f"if {from_} is not MISSING and {to} is not MISSING:"
+                f"if {from_} is not MISSING and {to} is not MISSING:",
                 f"    raise ValueError('The features \\'{from_}\\' and \\'{to}\\' are aliases of each other. Only one can be specified, but both were given.')",
                 f"{from_} = {to} if {from_} is MISSING else {from_}",
             ]
@@ -274,8 +278,8 @@ def _get_field(
     comments: Mapping[str, str],
     class_owner: Optional[str],
     class_tags: Optional[Tuple[str, ...]],
-    class_etl_offline_to_online: Optional[bool],
-    class_max_staleness: Optional[Duration],
+    class_etl_offline_to_online: bool,
+    class_max_staleness: timedelta,
     namespace: str,
 ) -> Feature:
     # Return a Field object for this field name and type.  ClassVars and
@@ -319,8 +323,8 @@ def _process_field(
     comments: Mapping[str, str],
     class_owner: Optional[str],
     class_tags: Optional[Tuple[str, ...]],
-    class_etl_offline_to_online: Optional[bool],
-    class_max_staleness: Optional[Duration],
+    class_etl_offline_to_online: bool,
+    class_max_staleness: timedelta,
 ) -> Feature:
     comment_for_feature = comments.get(f.attribute_name)
     comment_based_description = None
@@ -370,11 +374,13 @@ def _process_field(
     elif f.owner is None:
         f.owner = comment_based_owner or class_owner
 
-    if f.max_staleness is Ellipsis:
-        f.max_staleness = class_max_staleness if class_max_staleness is not Ellipsis else None
+    # Using the private variable because the max_staleness is a read-only property
+    if f._max_staleness is ...:
+        f._max_staleness = class_max_staleness
 
-    if f.etl_offline_to_online is None:
-        f.etl_offline_to_online = False if class_etl_offline_to_online is None else class_etl_offline_to_online
+    # Using the private variable because the etl_offline_to_online is a read-only property
+    if f._etl_offline_to_online is None:
+        f._etl_offline_to_online = False if class_etl_offline_to_online is None else class_etl_offline_to_online
     return f
 
 
@@ -489,8 +495,8 @@ def _process_class(
     cls: Type[T],
     owner: Optional[str],
     tags: Tuple[str, ...],
-    etl_offline_to_online: Optional[bool],
-    max_staleness: Optional[Duration],
+    etl_offline_to_online: bool,
+    max_staleness: timedelta,
     namespace: str,
 ) -> Type[T]:
     raw_cls_annotations = cls.__dict__.get("__annotations__", {})
@@ -581,7 +587,7 @@ def _process_class(
 
     set_new_attribute(cls=cls, name="__chalk_ts__", value=classproperty(__chalk_ts__, cached=True))
 
-    def __chalk_observed_at__(cls: Type[Features]) -> Feature:
+    def __chalk_observed_at__(cls: Type[Features]) -> FeatureWrapper:
         ts_feature: Optional[Feature] = _discover_feature(
             cls_fields,
             "feature time",
@@ -623,6 +629,7 @@ def _process_class(
     def __features__(cls: Features) -> List[Feature]:
         features = list(cls_fields)
         if cls.__chalk_ts__ not in features:
+            assert cls.__chalk_ts__ is not None
             features.append(cls.__chalk_ts__)
         return features
 
@@ -636,13 +643,11 @@ def _process_class(
     set_new_attribute(cls=cls, name="__chalk_namespace__", value=namespace)
     set_new_attribute(cls=cls, name="__chalk_owner__", value=owner)
     set_new_attribute(cls=cls, name="__chalk_tags__", value=list(tags))
-    set_new_attribute(
-        cls=cls, name="__chalk_max_staleness__", value=max_staleness if max_staleness is not Ellipsis else None
-    )
+    set_new_attribute(cls=cls, name="__chalk_max_staleness__", value=max_staleness)
     set_new_attribute(
         cls=cls,
         name="__chalk_etl_offline_to_online__",
-        value=etl_offline_to_online if etl_offline_to_online is not Ellipsis else False,
+        value=etl_offline_to_online,
     )
     set_new_attribute(cls=cls, name="__is_features__", value=True)
     set_new_attribute(cls=cls, name="__len__", value=_len_fn(_globals=_globals))
@@ -692,10 +697,12 @@ def _process_class(
             if f_i.attribute_name in cls_annotations:
                 assert f_i.features_cls is not None
                 raise ValueError(
-                    f"The class '{f_i.features_cls.__name__}' "
-                    f"has an existing annotation '{f_i.attribute_name}' "
-                    "that collides with a versioned feature. Please remove the existing "
-                    "annotation, or lower the version."
+                    (
+                        f"The class '{f_i.features_cls.__name__}' "
+                        f"has an existing annotation '{f_i.attribute_name}' "
+                        "that collides with a versioned feature. Please remove the existing "
+                        "annotation, or lower the version."
+                    )
                 )
 
     set_new_attribute(
