@@ -161,7 +161,7 @@ class FeatureWrapper:
 
         # Passing through __getattr__ on has_one features, as users can use getattr
         # notation in annotations for resolvers
-        if item.startswith("__"):
+        if item.startswith("__") and not item.startswith("__chalk"):
             # Short-circuiting on the dunders to be compatible with copy.copy
             raise AttributeError(item)
 
