@@ -9,9 +9,6 @@ if TYPE_CHECKING:
     from chalk.features.feature_field import Feature
 
 
-__all__ = ["Filter", "before", "after", "TimeDelta"]
-
-
 class Filter:
     def __init__(self, lhs: Any, operation: str, rhs: Any):
         from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
@@ -234,3 +231,11 @@ def after(
             microseconds_ago=microseconds_ago,
         ),
     )
+
+
+__all__ = [
+    "Filter",
+    "before",
+    "after",
+    "TimeDelta",
+]
