@@ -25,7 +25,7 @@ def _py_path_to_module(path: Path, repo_root: Path) -> str:
     return str(p)[: -len(".py")].replace("./", "").replace("/", ".")
 
 
-def _import_all_files() -> List[FailedImport]:
+def import_all_files() -> List[FailedImport]:
     project_root: Optional[Path] = get_directory_root()
     if project_root is None:
         return [
