@@ -1,6 +1,6 @@
 import dataclasses
 import json
-from typing import Any, List, Type, TypeVar
+from typing import Any, List, Type, TypeVar, overload
 
 import pydantic
 from pydantic.main import BaseModel
@@ -37,6 +37,7 @@ from chalk.streams.types import (
     StreamResolverParamMessageWindow,
 )
 from chalk.utils import paths
+from chalk.utils.duration import timedelta_to_duration
 from chalk.utils.string import to_snake_case
 
 T = TypeVar("T")
@@ -93,6 +94,7 @@ def _convert_type(underlying_type: Type) -> _ConvertedType:
 
 
 def _get_feature_id(s: Feature):
+    assert s.features_cls is not None
     assert s.name is not None, "Could not find feature name"
     assert s.namespace is not None, "Could not find feature namespace"
     return UpsertFeatureIdGQL(
@@ -121,7 +123,42 @@ def _get_path_component(pc: HasOnePathObj) -> UpsertReferencePathComponentGQL:
     )
 
 
-def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
+@overload
+def convert_type_to_gql(t: Filter) -> UpsertFilterGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: StreamResolver) -> UpsertStreamResolverGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: SinkResolver) -> UpsertSinkResolverGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: OnlineResolver) -> UpsertResolverGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: OfflineResolver) -> UpsertResolverGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: StreamResolverParam) -> UpsertStreamResolverParamGQL:
+    ...
+
+
+@overload
+def convert_type_to_gql(t: Feature) -> UpsertFeatureGQL:
+    ...
+
+
+def convert_type_to_gql(t: Any):
     if isinstance(t, StreamResolver):
         return UpsertStreamResolverGQL(
             fqn=t.fqn,
@@ -162,8 +199,8 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
             doc=t.doc,
             machineType=t.machine_type,
             bufferSize=t.buffer_size,
-            debounce=t.debounce,
-            maxDelay=t.max_delay,
+            debounce=None if t.debounce is None else timedelta_to_duration(t.debounce),
+            maxDelay=None if t.max_delay is None else timedelta_to_duration(t.max_delay),
             upsert=t.upsert,
             owner=t.owner,
         )
@@ -171,7 +208,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
     if isinstance(t, (OnlineResolver, OfflineResolver)):
         cron = t.cron
         if isinstance(cron, Cron):
-            assert t.cron.schedule is not None, "`Cron` object must be constructed with a `schedule` property."
+            assert cron.schedule is not None, "`Cron` object must be constructed with a `schedule` property."
             cron = cron.schedule
         inputs = []
         all_inputs = []
@@ -186,6 +223,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
             elif isinstance(f, type) and issubclass(f, DataFrame):
                 all_inputs.append(UpsertResolverInputUnionGQL(dataframe=_convert_df(f)))
 
+        assert t.output is not None
         return UpsertResolverGQL(
             fqn=t.fqn,
             kind="offline" if isinstance(t, OfflineResolver) else "online",
@@ -208,7 +246,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
             cron=cron,
             machineType=t.machine_type,
             owner=t.owner,
-            timeout=t.timeout,
+            timeout=None if t.timeout is None else timedelta_to_duration(t.timeout),
         )
 
     if isinstance(t, StreamResolverParam):
@@ -234,6 +272,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
                 default_value = json.loads(json.dumps(dataclasses.asdict(t.default_value)))
                 schema = pydantic.dataclasses.dataclass(t.typ).__pydantic_model__.schema()
             elif isinstance(t.default_value, BaseModel):
+                assert issubclass(t.typ, BaseModel)
                 default_value = json.loads(t.default_value.json())
                 schema = json.loads(t.typ.schema_json())
             return UpsertStreamResolverParamGQL(
@@ -250,6 +289,7 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
     if isinstance(t, Feature):
         assert t.name is not None, "Feature has no name"
         assert t.namespace is not None, "Feature has no namespace"
+        assert t.features_cls is not None
         scalar_kind_gql = None
         has_one_kind_gql = None
         has_many_kind_gql = None
@@ -257,8 +297,10 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
         if t.is_autogenerated:
             raise RuntimeError("Autogenerated features should not be converted")
         if t.is_has_one:
+            assert t.join is not None
             has_one_kind_gql = UpsertHasOneKindGQL(join=convert_type_to_gql(t.join))
         elif t.is_has_many:
+            assert t.join is not None
             has_many_kind_gql = UpsertHasManyKindGQL(
                 join=convert_type_to_gql(t.join),
                 columns=None,
@@ -292,10 +334,10 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
                 attributeName=t.attribute_name,
                 explicitNamespace=t.namespace != to_snake_case(t.features_cls.__name__),
             ),
-            maxStaleness=t.max_staleness,
+            maxStaleness=timedelta_to_duration(t.max_staleness),
             description=t.description,
             owner=t.owner,
-            windowBuckets=t.window_buckets and sorted(list(t.window_buckets)),
+            windowBuckets=None if t.window_buckets is None else sorted(list(t.window_buckets)),
             etlOfflineToOnline=t.etl_offline_to_online,
             tags=t.tags,
             hasOneKind=has_one_kind_gql,
@@ -322,4 +364,4 @@ def convert_type_to_gql(t: Any) -> dataclasses.dataclass:
             rhs=_get_feature_id(t.rhs),
         )
 
-    return t
+    raise ValueError(f"Unable to convert {t} to GQL")
