@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import os
-from typing import TYPE_CHECKING, Any, Mapping, Optional
+from typing import TYPE_CHECKING, Any, Dict, Optional
 
 from chalk.sql._internal.sql_source import BaseSQLSource
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -33,7 +33,7 @@ class CloudSQLSourceImpl(BaseSQLSource):
 
         BaseSQLSource.__init__(self, name=name)
 
-    def engine_args(self) -> Mapping[str, Any]:
+    def engine_args(self) -> Dict[str, Any]:
         return dict(
             pool_size=20,
             max_overflow=60,
