@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Dict, Mapping, Optional, Union
+from typing import TYPE_CHECKING, Any, Dict, Optional, Union
 
 from chalk.integrations.named import load_integration_variable
 from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn
@@ -45,8 +45,9 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
             max_overflow=60,
             pool_pre_ping=True,
         )
-        additional = self.load_env_engine_args(name=self.name)
-        default.update(additional)
+        if self.name is not None:
+            additional = self.load_env_engine_args(name=self.name)
+            default.update(additional)
         return default
 
     def local_engine_url(self) -> URL:
