@@ -5,7 +5,7 @@ import csv
 import io
 import logging
 import os
-from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
 
 import pyarrow as pa
 import pyarrow.csv
@@ -92,7 +92,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         default = dict(
             pool_size=20,
             max_overflow=60,
-            pool_pre_ping=os.getenv('USE_CLIENT_POOL_PRE_PING', 'true') == 'true',
+            pool_pre_ping=env_var_bool("USE_CLIENT_POOL_PRE_PING"),
             # Trying to fix mysterious dead connection issue
             connect_args={
                 "keepalives": 1,
@@ -101,8 +101,9 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
                 "keepalives_count": 5,
             },
         )
-        additional = self.load_env_engine_args(name=self.name)
-        default.update(additional)
+        if self.name is not None:
+            additional = self.load_env_engine_args(name=self.name)
+            default.update(additional)
         return default
 
     def local_engine_url(self) -> URL:
