@@ -1,9 +1,13 @@
-from typing import Optional
+from __future__ import annotations
 
-from sqlalchemy.engine.url import URL
+from typing import TYPE_CHECKING, Optional
 
 from chalk.integrations.named import load_integration_variable
 from chalk.sql._internal.sql_source import BaseSQLSource
+from chalk.utils.missing_dependency import missing_dependency_exception
+
+if TYPE_CHECKING:
+    from sqlalchemy.engine.url import URL
 
 
 class RedshiftSourceImpl(BaseSQLSource):
@@ -15,6 +19,11 @@ class RedshiftSourceImpl(BaseSQLSource):
         password: Optional[str] = None,
         name: Optional[str] = None,
     ):
+        try:
+            import redshift_connector
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[redshift]")
+        del redshift_connector
         self.host = host or load_integration_variable(name="REDSHIFT_HOST", integration_name=name)
         self.db = db or load_integration_variable(name="REDSHIFT_DB", integration_name=name)
         self.user = user or load_integration_variable(name="REDSHIFT_USER", integration_name=name)
@@ -22,6 +31,8 @@ class RedshiftSourceImpl(BaseSQLSource):
         BaseSQLSource.__init__(self, name=name)
 
     def local_engine_url(self) -> URL:
+        from sqlalchemy.engine.url import URL
+
         return URL.create(
             drivername="redshift+psycopg2",
             username=self.user,
