@@ -1,10 +1,15 @@
-from os import PathLike
-from typing import Any, Dict, Optional, TypeVar, Union
+from __future__ import annotations
 
-from sqlalchemy.engine.url import URL
+from os import PathLike
+from typing import TYPE_CHECKING, Any, Dict, Optional, TypeVar, Union
 
 from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn
 from chalk.sql.protocols import SQLSourceWithTableIngestProtocol
+from chalk.utils.missing_dependency import missing_dependency_exception
+
+if TYPE_CHECKING:
+    from sqlalchemy.engine.url import URL
+
 
 T = TypeVar("T")
 
@@ -15,6 +20,10 @@ class SQLiteInMemorySourceImpl(TableIngestMixIn, BaseSQLSource, SQLSourceWithTab
         BaseSQLSource.__init__(self, name=name)
 
     def local_engine_url(self) -> URL:
+        try:
+            from sqlalchemy.engine.url import URL
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[sqlite]")
         return URL.create(drivername="sqlite", database=":memory:", query={"check_same_thread": "true"})
 
 
@@ -25,4 +34,8 @@ class SQLiteFileSourceImpl(TableIngestMixIn, BaseSQLSource, SQLSourceWithTableIn
         BaseSQLSource.__init__(self, name=name)
 
     def local_engine_url(self) -> URL:
+        try:
+            from sqlalchemy.engine.url import URL
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[sqlite]")
         return URL.create(drivername="sqlite", database=self.filename, query={"check_same_thread": "true"})
