@@ -6,7 +6,7 @@ import json
 import os
 import re
 from pathlib import Path
-from typing import TYPE_CHECKING, Dict, Iterable, List, Literal, Optional, Type, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Literal, Optional, Type, Union, cast
 
 import yaml
 from pydantic import BaseModel, ValidationError, validator
@@ -14,7 +14,6 @@ from pydantic import BaseModel, ValidationError, validator
 from chalk import OfflineResolver, OnlineResolver
 from chalk.features import DataFrame, Feature, Features, FeatureSetBase
 from chalk.features.resolver import ResolverArgErrorHandler, StreamResolver
-from chalk.sql import TableIngestProtocol
 from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
 from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
 from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
@@ -45,7 +44,7 @@ _SOURCES = {
     "kafka": KafkaSource,
 }
 
-_DIALECTS_SQLGLOT = {
+_DIALECTS_SQLGLOT: Dict[Type[BaseSQLSource], str] = {
     SnowflakeSourceImpl: "snowflake",
     PostgreSQLSourceImpl: "postgres",
     MySQLSourceImpl: "mysql",
@@ -71,7 +70,9 @@ class IncrementalSettings(BaseModel):
 
     @validator("lookback_period")
     @classmethod
-    def validate_lookback_period(cls, value):
+    def validate_lookback_period(cls, value: Optional[str]):
+        if value is None:
+            return None
         try:
             parse_chalk_duration(value)
         except Exception as e:
@@ -80,7 +81,7 @@ class IncrementalSettings(BaseModel):
 
     @validator("mode")
     @classmethod
-    def validate_mode(cls, mode, values):
+    def validate_mode(cls, mode: Literal["row", "group", "parameter"], values: Dict[str, Any]):
         if mode in ["row", "group"] and not values["incremental_column"]:
             raise ValueError(f"'incremental_column' must be set if mode is 'row' or 'group'.")
         return mode
@@ -92,7 +93,7 @@ class CommentDict(BaseModel):
     incremental: Optional[IncrementalSettings]
     tags: Optional[List[str]]
     environment: Optional[List[str]]
-    count: Optional[Literal[1, "one"]]
+    count: Optional[Literal[1, "one", "one_or_none"]]
     cron: Optional[str]
     machine_type: Optional[str]
     max_staleness: Optional[str]
@@ -103,7 +104,7 @@ class CommentDict(BaseModel):
 
     @validator("tags", "environment", pre=True)
     @classmethod
-    def validate_list_inputs(cls, value):
+    def validate_list_inputs(cls, value: Union[str, List[str], None]):
         if isinstance(value, list):
             return value
         if isinstance(value, str):
@@ -112,7 +113,9 @@ class CommentDict(BaseModel):
 
     @validator("cron", "max_staleness", "timeout")
     @classmethod
-    def validate_timedelta(cls, string):
+    def validate_timedelta(cls, string: Optional[str]):
+        if string is None:
+            return None
         try:
             parse_chalk_duration(string)
         except Exception as e:
@@ -121,16 +124,18 @@ class CommentDict(BaseModel):
 
     @validator("type")
     @classmethod
-    def validate_type(cls, resolver_type):
+    def validate_type(cls, resolver_type: str):
         if resolver_type not in _RESOLVER_TYPES:
             raise ValueError(
-                f"Resolver type '{resolver_type}' not supported. "
-                f"'online', 'offline' and 'streaming' are supported options"
+                (
+                    f"Resolver type '{resolver_type}' not supported. "
+                    f"'online', 'offline' and 'streaming' are supported options"
+                )
             )
         return resolver_type
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class ResolverError:
     """Generic class for returning errors at any point during resolution process"""
 
@@ -139,18 +144,18 @@ class ResolverError:
     parameter: Optional[str]
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class ResolverResult:
     """Chief return class with resolver we actually use"""
 
-    resolver: Optional[OnlineResolver]
+    resolver: Optional[Union[OnlineResolver, OfflineResolver, StreamResolver]]
     errors: List[ResolverError]
-    db: Optional[TableIngestProtocol]
+    db: Optional[Union[BaseSQLSource, StreamSource]]
     fields: Optional[Dict[str, str]]
     args: Optional[Dict[str, str]]
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class SQLStringResult:
     """Class for getting the sql string from the file"""
 
@@ -163,28 +168,28 @@ class SQLStringResult:
         return cls(path=path, sql_string=None, error=ResolverError(display=display_error, path=path, parameter=None))
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class GlotResult:
     """Class for editing the sql string, and using sqlglot on sql string"""
 
     sql_string: str
     glot: Optional[Expression]
-    args: Optional[Dict[str, str]]
+    args: Dict[str, str]
     default_args: List[Optional[str]]
     comment_dict: Optional[CommentDict]
     docstring: Optional[str]
     errors: List[ResolverError]
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class ParseResult:
     """Class for important info gathered from glot"""
 
     sql_string: str
-    comment_dict: Optional[CommentDict]
-    fields: Optional[Dict[str, str]]
-    namespace: Optional[str]
-    source: Optional[Union[BaseSQLSource, StreamSource]]
+    comment_dict: CommentDict
+    fields: Dict[str, str]
+    namespace: str
+    source: Union[BaseSQLSource, StreamSource, None]
     docstring: Optional[str]
     errors: List[ResolverError]
 
@@ -194,6 +199,7 @@ def get_sql_file_resolvers(
 ) -> Iterable[ResolverResult]:
     """Iterate through all `.chalk.sql` filepaths, gather the sql strings, and get a resolver hopefully for each."""
     for dp, dn, fn in os.walk(os.path.expanduser(sql_file_resolve_location)):
+        del dn  # unused
         for f in fn:
             filepath = os.path.join(dp, f)
             if not filepath.endswith(".chalk.sql"):
@@ -206,14 +212,15 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
     """Parse the sql strings and get a ResolverResult from each"""
     if sql_string_result.error:
         return ResolverResult(resolver=None, errors=[sql_string_result.error], db=None, fields=None, args=None)
+    assert sql_string_result.sql_string is not None
     path = sql_string_result.path
 
     errors: List[ResolverError] = []
-    glot_result: GlotResult = _get_sql_glot(sql_string=sql_string_result.sql_string, path=path, sources=sources)
+    glot_result = _get_sql_glot(sql_string=sql_string_result.sql_string, path=path, sources=sources)
     if glot_result.errors:
         return ResolverResult(resolver=None, errors=glot_result.errors, db=None, fields=None, args=None)
 
-    parsed: ParseResult = _parse_glot(glot_result, path, sources)
+    parsed = _parse_glot(glot_result, path, sources)
     if parsed.errors:
         return ResolverResult(resolver=None, errors=parsed.errors, db=parsed.source, fields=None, args=None)
 
@@ -253,22 +260,21 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
     resolver_type = _RESOLVER_TYPES[resolver_type_str]
 
     if resolver_type == StreamResolver:
-        result: ResolverResult = _get_stream_resolver(path, glot_result, parsed, outputs)
-        return result
-
-    result = ResolverResult(resolver=None, errors=errors, db=parsed.source, fields=None, args=None)
+        return _get_stream_resolver(path, glot_result, parsed, outputs)
 
     incremental_dict = parsed.comment_dict.incremental.dict() if parsed.comment_dict.incremental else None
     return_one = parsed.comment_dict.count
+    source = parsed.source
+    assert isinstance(source, BaseSQLSource)
 
     # function for online resolver to process
     def fn(
-        *input_values,
-        database=parsed.source,
-        sql_query=parsed.sql_string,
-        field_dict=parsed.fields,
-        args_dict=glot_result.args,
-        incremental=incremental_dict,
+        *input_values: Any,
+        database: BaseSQLSource = source,
+        sql_query: str = parsed.sql_string,
+        field_dict: Dict[str, str] = parsed.fields,
+        args_dict: Dict[str, str] = glot_result.args,
+        incremental: Optional[Dict[str, Any]] = incremental_dict,
     ):
         arg_dict = {arg: input_value for input_value, arg in zip(input_values, args_dict.keys())}
         func = database.query_string(
@@ -278,19 +284,25 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
         )
         if incremental:
             func = func.incremental(**incremental)
-        if return_one:
+        elif return_one in [1, "one"]:
             func = func.one()
+        elif return_one == "one_or_none":
+            func = func.one_or_none()
         return func
 
-    if result.errors:
-        return result
+    if errors:
+        return ResolverResult(
+            resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
+        )
 
     if return_one:
         output = Features[tuple(outputs)]
     else:
         output = Features[DataFrame[tuple(outputs)]]
 
-    default_args = [ResolverArgErrorHandler(default_value) for default_value in glot_result.default_args]
+    default_args: List[Optional[ResolverArgErrorHandler]] = [
+        ResolverArgErrorHandler(default_value) for default_value in glot_result.default_args
+    ]
 
     filename = os.path.basename(path)
     # attempt to instantiate the resolver
@@ -316,24 +328,25 @@ def get_sql_file_resolver(sources: Iterable[BaseSQLSource], sql_string_result: S
             timeout=parsed.comment_dict.timeout,
         )
     except Exception as e:
-        result.errors.append(
+        errors.append(
             ResolverError(
                 display=f"{resolver_type_str.capitalize()} resolver could not be instantiated, {e}",
                 path=path,
                 parameter=None,
             )
         )
-        return result
+        return ResolverResult(
+            resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
+        )
 
-    result.resolver = resolver
-    result.fields = parsed.fields
-    result.args = glot_result.args
-    return result
+    return ResolverResult(
+        resolver=resolver, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
+    )
 
 
 def _get_sql_string(path: str) -> SQLStringResult:
     """Attempt to get a sql string from a filepath and gracefully exit if unable to"""
-    sql_string_result = SQLStringResult(path=path, sql_string=None, error=None)
+
     if not path.endswith(".chalk.sql"):
         return SQLStringResult.fail(display_error=f"sql resolver file '{path}' must end in '.chalk.sql'", path=path)
     sql_string = None
@@ -351,8 +364,7 @@ def _get_sql_string(path: str) -> SQLStringResult:
                 sql_string = f.read()
     if sql_string is None:
         return SQLStringResult.fail(display_error=f"Cannot find file '{path}'", path=path)
-    sql_string_result.sql_string = sql_string
-    return sql_string_result
+    return SQLStringResult(path=path, sql_string=sql_string, error=None)
 
 
 def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource]) -> GlotResult:
@@ -362,11 +374,10 @@ def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource])
         import sqlglot.expressions
     except ImportError:
         raise missing_dependency_exception("chalkpy[runtime]")
-    glot_result = GlotResult(
-        sql_string=sql_string, glot=None, args=None, default_args=[], comment_dict=None, docstring=None, errors=[]
-    )
     args = {}  # sql string -> input feature string
     variables = set(re.findall("\\${.*?\\}", sql_string))
+    errors: List[ResolverError] = []
+    default_args: List[Optional[str]] = []
     # replace ?{variable_name} with :variable_name for sqlalchemy, and keep track of input args necessary
     for variable_pattern in variables:
         has_default_arg = False
@@ -376,10 +387,12 @@ def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource])
             if split_var in variable:
                 split = variable.split(split_var)
                 if len(split) != 2:
-                    glot_result.errors.append(
+                    errors.append(
                         ResolverError(
-                            display=f"If character '|' is used, both variable name and default value must be "
-                            f'specified in ({variable}) like \'?{{variable_name | "default_value"}}',
+                            display=(
+                                f"If character '|' is used, both variable name and default value must be "
+                                f'specified in ({variable}) like \'?{{variable_name | "default_value"}}'
+                            ),
                             path=path,
                             parameter=None,
                         )
@@ -390,22 +403,18 @@ def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource])
                     default_arg = json.loads(default_arg)
                     f = Feature.from_root_fqn(variable)
                     default_arg = f.converter.from_json_to_rich(default_arg)
-                    glot_result.default_args.append(default_arg)
+                    default_args.append(default_arg)
                     has_default_arg = True
         if not has_default_arg:
             # TODO default value of None should be refactored since None is a legitimate default value
-            glot_result.default_args.append(None)
+            default_args.append(None)
         period_replaced = variable.replace(".", "_")
         sql_safe_str = f"__chalk_{period_replaced}__"
         sql_string = sql_string.replace(variable_pattern, f":{sql_safe_str}")
         args[sql_safe_str] = variable
-
-    glot_result.sql_string = sql_string
-    glot_result.args = args
-
     comments = ""
     docstring = ""
-    for comment in glot_result.sql_string.splitlines():
+    for comment in sql_string.splitlines():
         if comment.strip().startswith("--"):
             comment = comment.strip().replace("--", "")
             if comment.strip().startswith("-"):
@@ -420,32 +429,47 @@ def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource])
             break
 
     try:
-        comment_dict = yaml.safe_load(comments)
+        comment_dict: Dict[str, Any] = yaml.safe_load(comments)
     except Exception as e:
-        glot_result.errors.append(
+        errors.append(
             ResolverError(
                 display=f"Comments key-values '{comments}' must be of YAML form, {e}",
                 path=path,
                 parameter=comments,
             )
         )
-        return glot_result
+        return GlotResult(
+            sql_string=sql_string,
+            glot=None,
+            args=args,
+            default_args=default_args,
+            comment_dict=None,
+            docstring=docstring,
+            errors=errors,
+        )
     try:
         comment_dict_object = CommentDict.parse_obj(comment_dict)
     except ValidationError as e:
         for error in e.errors():
-            glot_result.errors.append(
+            errors.append(
                 ResolverError(
-                    display=f"Could not parse comment(s) '{'.'.join(error['loc'])}': {error['msg']}",
+                    display=f"Could not parse comment(s) '{'.'.join(str(x) for x in error['loc'])}': {error['msg']}",
                     path=path,
-                    parameter=comment_dict,
+                    parameter=json.dumps(comment_dict),
                 )
             )
-        return glot_result
-    glot_result.comment_dict = comment_dict_object
-    glot_result.docstring = docstring.strip()
+        return GlotResult(
+            sql_string=sql_string,
+            glot=None,
+            args=args,
+            default_args=default_args,
+            comment_dict=None,
+            docstring=docstring,
+            errors=errors,
+        )
+    docstring = docstring.strip()
 
-    source_comment = glot_result.comment_dict.source
+    source_comment = comment_dict_object.source
     source_type = None
     if source_comment not in _SOURCES:  # actual name of source
         for possible_source in sources:
@@ -455,92 +479,106 @@ def _get_sql_glot(sql_string: str, path: str, sources: Iterable[BaseSQLSource])
         if source_comment in _DIALECTS_SQLGLOT.values():  # source type, if supported
             source_type = source_comment
     try:
-        glot_result.glot = sqlglot.parse_one(sql=glot_result.sql_string, read=source_type)
+        glot = sqlglot.parse_one(sql=sql_string, read=source_type)
     except Exception as e:
-        glot_result.errors.append(
-            ResolverError(display=f"Cannot SQL parse {glot_result.sql_string}, {e}", path=path, parameter=None)
+        errors.append(ResolverError(display=f"Cannot SQL parse {sql_string}, {e}", path=path, parameter=None))
+        return GlotResult(
+            sql_string=sql_string,
+            glot=None,
+            args=args,
+            default_args=default_args,
+            comment_dict=comment_dict_object,
+            docstring=docstring,
+            errors=errors,
         )
-        return glot_result
-    if not isinstance(glot_result.glot, sqlglot.expressions.Select):
-        glot_result.errors.append(
-            ResolverError(
-                display=f"SQL query {glot_result.sql_string} should be of 'SELECT' type", path=path, parameter=None
-            )
+    if not isinstance(glot, sqlglot.expressions.Select):
+        errors.append(
+            ResolverError(display=f"SQL query {sql_string} should be of 'SELECT' type", path=path, parameter=None)
         )
-    return glot_result
+    return GlotResult(
+        sql_string=sql_string,
+        glot=glot,
+        args=args,
+        default_args=default_args,
+        comment_dict=comment_dict_object,
+        docstring=docstring,
+        errors=errors,
+    )
 
 
 def _parse_glot(glot_result: GlotResult, path: str, sources: Iterable[BaseSQLSource]) -> ParseResult:
     """Parse useful info from sqlglot and gracefully exit if unable to"""
-    parse_result = ParseResult(
-        sql_string=glot_result.sql_string,
-        comment_dict=None,
-        fields=None,
-        namespace=None,
-        source=None,
-        docstring=None,
-        errors=[],
-    )
-
     # define a source SQL database. Can either specify name or kind if only one of the kind is present.
-    parse_result.comment_dict = glot_result.comment_dict
-    parse_result.docstring = glot_result.docstring
-    source_name = parse_result.comment_dict.source
+    comment_dict = glot_result.comment_dict
+    assert comment_dict is not None
+    docstring = glot_result.docstring
+    source_name = comment_dict.source
     source = None
+    errors: List[ResolverError] = []
     if source_name not in _SOURCES:  # actual name of source
         for possible_source in sources:
             if possible_source.name == source_name:
                 source = possible_source
     else:
         for possible_source in sources:
-            if isinstance(possible_source, _SOURCES.get(source_name)):
+            if isinstance(possible_source, _SOURCES[source_name]):
                 if source:
-                    parse_result.errors.append(
+                    errors.append(
                         ResolverError(
-                            display=f"More than one {source_name} source exists. Instead, refer to the integration by "
-                            f"name among ({[source.name for source in sources]}).",
+                            display=(
+                                f"More than one {source_name} source exists. Instead, refer to the integration by "
+                                f"name among ({[source.name for source in sources]})."
+                            ),
                             path=path,
                             parameter=source_name,
                         )
                     )
                 source = possible_source
     if not source:
-        parse_result.errors.append(
-            ResolverError(display=f"Source '{source_name}' not found", path=path, parameter=source_name)
-        )
-    parse_result.source = source
-
-    if parse_result.errors:
-        return parse_result
-
+        errors.append(ResolverError(display=f"Source '{source_name}' not found", path=path, parameter=source_name))
     # get resolver fields: which columns selected will match to which chalk feature?
-    namespace = parse_result.comment_dict.resolves
+    namespace = comment_dict.resolves
     if namespace not in FeatureSetBase.registry:
         camelcase_namespace = to_camel_case(namespace)
         if camelcase_namespace in FeatureSetBase.registry:
             namespace = camelcase_namespace
         else:
-            parse_result.errors.append(
+            errors.append(
                 ResolverError(
                     display=f"No @features class with the name '{namespace}'", path=path, parameter=source_name
                 )
             )
-    parse_result.namespace = namespace
-    fields = {}  # sql string -> output feature string
+    if errors:
+        return ParseResult(
+            sql_string=glot_result.sql_string,
+            comment_dict=comment_dict,
+            fields={},
+            namespace=namespace,
+            source=source,
+            docstring=docstring,
+            errors=errors,
+        )
 
+    fields: Dict[str, str] = {}  # sql string -> output feature string
+    assert glot_result.glot is not None
     for column_name in glot_result.glot.named_selects:
         fields[column_name] = f"{namespace}.{column_name}"
-    parse_result.fields = fields
 
-    return parse_result
+    return ParseResult(
+        sql_string=glot_result.sql_string,
+        comment_dict=comment_dict,
+        fields=fields,
+        namespace=namespace,
+        source=source,
+        docstring=docstring,
+        errors=errors,
+    )
 
 
 def _get_stream_resolver(
     path: str, glot_result: GlotResult, parsed: ParseResult, outputs: List[Feature]
 ) -> ResolverResult:
     errors = []
-    result = ResolverResult(resolver=None, errors=errors, db=parsed.source, fields=None, args=None)
-
     output_features = Features[DataFrame[tuple(outputs)]]
 
     if isinstance(output_features.features[0], type) and issubclass(output_features.features[0], DataFrame):
@@ -555,6 +593,7 @@ def _get_stream_resolver(
 
     sql_query: str = _remove_comments(parsed.sql_string)
     filename = os.path.basename(path)
+    assert isinstance(parsed.source, StreamSource)
 
     try:
 
@@ -582,19 +621,18 @@ def _get_stream_resolver(
             parse=None,
         )
     except Exception as e:
-        result.errors.append(
+        errors.append(
             ResolverError(
                 display=f"Streaming resolver could not be instantiated, {e}",
                 path=path,
                 parameter=None,
             )
         )
-        return result
+        return ResolverResult(resolver=None, errors=errors, db=parsed.source, fields=None, args=None)
 
-    result.resolver = resolver
-    result.fields = parsed.fields
-    result.args = glot_result.args
-    return result
+    return ResolverResult(
+        resolver=resolver, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
+    )
 
 
 def _remove_comments(sql_string: str) -> str:
