@@ -146,7 +146,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self,
         path: Union[str, bytes, PathLike],
         fields: Optional[Mapping[str, Union[Feature, str, Any]]] = None,
-        args: Optional[Mapping[str, str]] = None,
+        args: Optional[Mapping[str, object]] = None,
     ) -> StringChalkQueryProtocol:
         sql_string = None
         fqn = None
@@ -189,7 +189,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self,
         query: str,
         fields: Optional[Mapping[str, Union[Feature, Any]]] = None,
-        args: Optional[Mapping[str, str]] = None,
+        args: Optional[Mapping[str, object]] = None,
     ) -> StringChalkQueryProtocol:
         fields = fields or {}
         fields = {f: unwrap_feature(v) if isinstance(v, FeatureWrapper) else v for (f, v) in fields.items()}
@@ -314,7 +314,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             **args,
         )
 
-    @functools.lru_cache(None)  # pyright: ignore[reportIncompatibleMethodOverride]
+    @functools.lru_cache(None)
     def _get_engine_default(self) -> Engine:
         from sqlalchemy.engine import create_engine
 
@@ -341,6 +341,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
 
 def _extract_features(feature_set: Features, prefix: str, targets: List[Label], features: Dict[str, Feature]):
     import sqlalchemy.sql.functions
+    import sqlalchemy.sql.schema
     from sqlalchemy import column
     from sqlalchemy.orm import InstrumentedAttribute
 
