@@ -6,7 +6,6 @@ from typing import (
     Any,
     Callable,
     Dict,
-    Iterator,
     List,
     Literal,
     Mapping,
@@ -416,7 +415,7 @@ class BaseSQLSourceProtocol(Protocol):
         self,
         query: str,
         fields: Optional[Mapping[str, Union[Feature, str, Any]]] = None,
-        args: Optional[Mapping[str, str]] = None,
+        args: Optional[Mapping[str, object]] = None,
     ) -> StringChalkQueryProtocol:
         """Run a query from a SQL string.
 
@@ -441,7 +440,7 @@ class BaseSQLSourceProtocol(Protocol):
         self,
         path: Union[str, bytes, PathLike],
         fields: Optional[Mapping[str, Union[Feature, str, Any]]] = None,
-        args: Optional[Mapping[str, str]] = None,
+        args: Optional[Mapping[str, object]] = None,
     ) -> StringChalkQueryProtocol:
         """Run a query from a SQL file.
 
@@ -512,22 +511,6 @@ class BaseSQLSourceProtocol(Protocol):
     def raw_session(self) -> Session:
         ...
 
-    def execute_query_to_pyarrow_tables(
-        self,
-        finalized_query: FinalizedChalkQuery,
-        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[Any] = None,
-    ) -> Iterator[pa.Table]:
-        ...
-
-    def execute_query_to_batches(
-        self,
-        finalized_query: FinalizedChalkQuery,
-        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[Any] = None,
-    ) -> List[pa.Table]:
-        ...
-
     def execute_query(
         self,
         finalized_query: FinalizedChalkQuery,
