@@ -1,6 +1,8 @@
-from typing import Any, List, Sequence, Set, Type, Union
+from __future__ import annotations
+
+import dataclasses
+from typing import TYPE_CHECKING, Any, List, Sequence, Set, Type, Union
 
-import polars
 import pyarrow
 from pydantic import BaseModel
 from typing_extensions import TypeAlias
@@ -8,11 +10,16 @@ from typing_extensions import TypeAlias
 from chalk.features import DataFrame
 from chalk.utils import AnyDataclass
 
+if TYPE_CHECKING:
+    import polars as pl
 
-class StreamResolverParam(BaseModel):
+
+@dataclasses.dataclass
+class StreamResolverParam:
     name: str
 
 
+@dataclasses.dataclass
 class StreamResolverParamMessage(StreamResolverParam):
     typ: Union[Type[str], Type[bytes], Type[BaseModel], AnyDataclass]
 
@@ -23,21 +30,24 @@ StreamResolverWindowType: TypeAlias = Union[
     Type[List[BaseModel]],
     Type[AnyDataclass],
     Type[pyarrow.Table],
-    Type[polars.DataFrame],
+    Type["pl.DataFrame"],
     Type[DataFrame],
     Any,  # The annotation value is likely going to be a GenericAlias, which messes with pydantic's validation
 ]
 
 
+@dataclasses.dataclass
 class StreamResolverParamMessageWindow(StreamResolverParam):
     typ: StreamResolverWindowType
 
 
-class StreamResolverSignature(BaseModel):
+@dataclasses.dataclass
+class StreamResolverSignature:
     params: Sequence[StreamResolverParam]
     output_feature_fqns: Set[str]
 
 
+@dataclasses.dataclass
 class StreamResolverParamKeyedState(StreamResolverParam):
     typ: Union[Type[BaseModel], Type[AnyDataclass]]
     default_value: Any
